@@ -11,12 +11,13 @@
 //
 // See http://www.boost.org/libs/mpl for documentation.
 
-// $Source: /cvsroot/boost/boost/boost/mpl/key_type.hpp,v $
-// $Date: 2004/09/02 15:40:41 $
-// $Revision: 1.2 $
+// $Source$
+// $Date: 2004-12-14 23:34:44 +0100 (Tue, 14 Dec 2004) $
+// $Revision: 26508 $
 
 #include <boost/mpl/key_type_fwd.hpp>
 #include <boost/mpl/sequence_tag.hpp>
+#include <boost/mpl/apply_wrap.hpp>
 #include <boost/mpl/aux_/na_spec.hpp>
 #include <boost/mpl/aux_/lambda_support.hpp>
 
@@ -27,8 +28,9 @@ template<
     , typename BOOST_MPL_AUX_NA_PARAM(T)
     >
 struct key_type
-    : key_type_impl< typename sequence_tag<AssociativeSequence>::type >
-        ::template apply<AssociativeSequence,T>
+    : apply_wrap2< 
+          key_type_impl< typename sequence_tag<AssociativeSequence>::type >
+        , AssociativeSequence, T>
 {
     BOOST_MPL_AUX_LAMBDA_SUPPORT(2,key_type,(AssociativeSequence,T))
 };
