@@ -1,5 +1,5 @@
 
-// Copyright Aleksey Gurtovoy 2001-2004
+// Copyright Aleksey Gurtovoy 2001-2006
 //
 // Distributed under the Boost Software License, Version 1.0. 
 // (See accompanying file LICENSE_1_0.txt or copy at 
@@ -7,12 +7,14 @@
 //
 // See http://www.boost.org/libs/mpl for documentation.
 
-// $Source: /cvsroot/boost/boost/boost/mpl/list/aux_/include_preprocessed.hpp,v $
-// $Date: 2004/12/01 02:44:52 $
-// $Revision: 1.4.2.1 $
+// $Source$
+// $Date: 2006-11-09 02:44:00 +0100 (Thu, 09 Nov 2006) $
+// $Revision: 35938 $
 
 // NO INCLUDE GUARDS, THE HEADER IS INTENDED FOR MULTIPLE INCLUSION!
 
+#include <boost/mpl/aux_/config/workaround.hpp>
+
 #include <boost/preprocessor/cat.hpp>
 #include <boost/preprocessor/stringize.hpp>
 
@@ -20,7 +22,13 @@
     aux_/preprocessed/plain/BOOST_MPL_PREPROCESSED_HEADER \
 /**/
 
+#if BOOST_WORKAROUND(__IBMCPP__, BOOST_TESTED_AT(700))
+#   define AUX778076_INCLUDE_STRING BOOST_PP_STRINGIZE(boost/mpl/list/AUX778076_HEADER)
+#   include AUX778076_INCLUDE_STRING
+#   undef AUX778076_INCLUDE_STRING
+#else
 #   include BOOST_PP_STRINGIZE(boost/mpl/list/AUX778076_HEADER)
+#endif
 
 #   undef AUX778076_HEADER
 
