@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// Licence: GNU General Public License version 2
-// $Id: GAfit.cpp 322 2007-07-24 00:17:11Z wojdyr $
+// Licence: GNU General Public License ver. 2+
+// $Id: GAfit.cpp 503 2009-06-09 15:47:31Z wojdyr $
 
 #include <stdlib.h>
 #include <time.h>
@@ -22,10 +22,10 @@ using namespace std;
 GAfit::GAfit(Ftk* F)
    : Fit(F, "Genetic-Algorithms"),
      popsize (100), elitism(0),
-     mutation_type('u'), p_mutation(0.1), mutate_all_genes(false), 
+     mutation_type('u'), p_mutation(0.1), mutate_all_genes(false),
      mutation_strength(0.1), crossover_type('u'), p_crossover(0.3),
-     selection_type('r'), rank_scoring(false), tournament_size(2), 
-     window_size(-1), 
+     selection_type('r'), rank_scoring(false), tournament_size(2),
+     window_size(-1),
      linear_scaling_a(1.), linear_scaling_c(2.), linear_scaling_b (1.),
      std_dev_stop(0), iter_with_no_progresss_stop(0),
      autoplot_indiv_nr(-1),
@@ -35,7 +35,7 @@ GAfit::GAfit(Ftk* F)
     /*
     irpar["population-size"] = IntRange (&popsize, 2, 9999);
     irpar["steady-size"] = IntRange (&elitism, 0, 9999);
-    epar.insert(pair<string, Enum_string>("mutation-type", 
+    epar.insert(pair<string, Enum_string>("mutation-type",
                                Enum_string (Distrib_enum, &mutation_type)));
     fpar["p-mutation"] = &p_mutation;
     bpar["mutate-all-genes"] = &mutate_all_genes;
@@ -48,7 +48,7 @@ GAfit::GAfit(Ftk* F)
     Crossover_enum ['A'] = "arithmetic2";
     Crossover_enum ['g'] = "guaranteed-avg";
     /*
-    epar.insert(pair<string, Enum_string>("crossover-type", 
+    epar.insert(pair<string, Enum_string>("crossover-type",
                                Enum_string (Crossover_enum, &crossover_type)));
     fpar["p-crossover"] = &p_crossover;
     */
@@ -57,7 +57,7 @@ GAfit::GAfit(Ftk* F)
     Selection_enum ['s'] = "SRS";
     Selection_enum ['d'] = "DS";
     /*
-    epar.insert (pair<string, Enum_string>("selection-type", 
+    epar.insert (pair<string, Enum_string>("selection-type",
                                Enum_string (Selection_enum, &selection_type)));
     bpar["rank-scoring"] = &rank_scoring;
     irpar["tournament-size"] = IntRange (&tournament_size, 2, 999);
@@ -82,7 +82,7 @@ fp GAfit::init()
     vector<Individual>::iterator best = pop->begin();
     for (vector<Individual>::iterator i = pop->begin(); i != pop->end(); i++) {
         i->g.resize(na);
-        for (int j = 0; j < na; j++)  
+        for (int j = 0; j < na; j++)
             i->g[j] = draw_a_from_distribution(j);
         compute_wssr_for_ind (i);
         if (i->raw_score < best->raw_score)
@@ -94,9 +94,9 @@ fp GAfit::init()
 
 void GAfit::autoiter()
 {
-    wssr_before = compute_wssr(a_orig, datsums);
+    wssr_before = compute_wssr(a_orig, dmdm_);
     F->msg ("WSSR before starting GA: " + S(wssr_before));
-    assert (pop && opop); 
+    assert (pop && opop);
     if (elitism >= popsize) {
         F->warn ("hmm, now elitism >= popsize, setting elitism = 1");
         elitism = 1;
@@ -114,30 +114,30 @@ void GAfit::autoiter()
 
 void GAfit::compute_wssr_for_ind (vector<Individual>::iterator ind)
 {
-    ind->raw_score = compute_wssr(ind->g, datsums);
+    ind->raw_score = compute_wssr(ind->g, dmdm_);
     ind->generation = iter_nr;
 }
 
 void GAfit::autoplot_in_autoiter()
 {
-    const Individual& indiv = is_index(autoplot_indiv_nr, *pop) 
+    const Individual& indiv = is_index(autoplot_indiv_nr, *pop)
                                     ? (*pop)[autoplot_indiv_nr] : best_indiv;
     iteration_plot(indiv.g);
 }
 
 void GAfit::mutation()
-{   
+{
     for (vector<Individual>::iterator i = pop->begin(); i != pop->end(); i++) {
         if (mutate_all_genes) {
             if (rand() < RAND_MAX * p_mutation) {
-                for (int j = 0; j < na; j++)  
+                for (int j = 0; j < na; j++)
                     i->g[j] = draw_a_from_distribution(j, mutation_type,
                                                             mutation_strength);
                 compute_wssr_for_ind (i);
             }
         }
         else
-            for (int j = 0; j < na; j++)  
+            for (int j = 0; j < na; j++)
                 if (rand() < RAND_MAX * p_mutation) {
                     i->g[j] = draw_a_from_distribution(j, mutation_type,
                                                             mutation_strength);
@@ -148,7 +148,7 @@ void GAfit::mutation()
 
 void GAfit::crossover()
 {
-    for (vector<Individual>::iterator i = pop->begin(); i != pop->end(); i++) 
+    for (vector<Individual>::iterator i = pop->begin(); i != pop->end(); i++)
         if (rand() < RAND_MAX / 2 * p_crossover) {
             vector<Individual>::iterator i2 = pop->begin() + rand()%pop->size();
             switch (crossover_type) {
@@ -182,15 +182,15 @@ void GAfit::crossover()
         }
 }
 
-void GAfit::uniform_crossover (vector<Individual>::iterator c1, 
+void GAfit::uniform_crossover (vector<Individual>::iterator c1,
                                vector<Individual>::iterator c2)
 {
-    for (int i = 0; i < na; i++) 
+    for (int i = 0; i < na; i++)
         if (rand() % 2)
             swap(c1->g[i], c2->g[i]);
 }
 
-void GAfit::one_point_crossover (vector<Individual>::iterator c1, 
+void GAfit::one_point_crossover (vector<Individual>::iterator c1,
                                  vector<Individual>::iterator c2)
 {
     int p = rand() % na;
@@ -198,40 +198,40 @@ void GAfit::one_point_crossover (vector<Individual>::iterator c1,
             swap(c1->g[j], c2->g[j]);
 }
 
-void GAfit::two_points_crossover (vector<Individual>::iterator c1, 
+void GAfit::two_points_crossover (vector<Individual>::iterator c1,
                                   vector<Individual>::iterator c2)
 {
     int p1 = rand() % na;
     int p2 = rand() % na;
-    for (int j = min(p1, p2); j < max(p1, p2); j++) 
+    for (int j = min(p1, p2); j < max(p1, p2); j++)
             swap(c1->g[j], c2->g[j]);
 }
 
-void GAfit::arithmetic_crossover1 (vector<Individual>::iterator c1, 
+void GAfit::arithmetic_crossover1 (vector<Individual>::iterator c1,
                                    vector<Individual>::iterator c2)
 {
     fp a = rand_0_1();
     for (int j = 0; j < na; j++) {
-        c1->g[j] = a * c1->g[j] + (1 - a) * c2->g[j]; 
+        c1->g[j] = a * c1->g[j] + (1 - a) * c2->g[j];
         c2->g[j] = (1 - a) * c1->g[j] + a * c2->g[j]; ;
     }
 }
 
-void GAfit::arithmetic_crossover2 (vector<Individual>::iterator c1, 
+void GAfit::arithmetic_crossover2 (vector<Individual>::iterator c1,
                                    vector<Individual>::iterator c2)
 {
     for (int j = 0; j < na; j++) {
         fp a = rand_0_1();
-        c1->g[j] = a * c1->g[j] + (1 - a) * c2->g[j]; 
+        c1->g[j] = a * c1->g[j] + (1 - a) * c2->g[j];
         c2->g[j] = (1 - a) * c1->g[j] + a * c2->g[j]; ;
     }
 }
 
-void GAfit::guaranteed_avarage_crossover (vector<Individual>::iterator c1, 
+void GAfit::guaranteed_avarage_crossover (vector<Individual>::iterator c1,
                                           vector<Individual>::iterator c2)
 {
-    for (int j = 0; j < na; j++) 
-        c1->g[j] = c2->g[j] = (c1->g[j] + c2->g[j]) / 2; 
+    for (int j = 0; j < na; j++)
+        c1->g[j] = c2->g[j] = (c1->g[j] + c2->g[j]) / 2;
 }
 
 void GAfit::pre_selection()
@@ -272,25 +272,25 @@ void GAfit::post_selection()
         return;
     do_rank_scoring (opop);
     for (vector<Individual>::iterator i = opop->begin(); i != opop->end(); i++)
-       if (i->phase_2_score < elitism) 
-           pop->push_back (*i); 
+       if (i->phase_2_score < elitism)
+           pop->push_back (*i);
     assert (size(*pop) == popsize);
 }
 
-struct ind_raw_sc_cmp 
+struct ind_raw_sc_cmp
 {
     bool operator() (Individual* a, Individual* b) {
         return a->raw_score < b->raw_score;
     }
 };
-        
+
 void GAfit::do_rank_scoring(std::vector<Individual> *popp)
 {
     // rank in population is assigned to phase_2_score
     // e.g. 0 - the best, 1 - second, (popp.size() - 1) - worst
     static vector<Individual*> ind_p;
     ind_p.resize(popp->size());
-    for (unsigned int i = 0; i < popp->size(); i++) 
+    for (unsigned int i = 0; i < popp->size(); i++)
         ind_p[i] = &(*popp)[i];
     sort (ind_p.begin(), ind_p.end(), ind_raw_sc_cmp());
     for (unsigned int i = 0; i < popp->size(); i++)
@@ -305,10 +305,10 @@ void GAfit::roulette_wheel_selection(vector<int>& next)
         t += static_cast<unsigned int>
             ((*pop)[i].norm_score * RAND_MAX / size(*pop));
         roulette[i] = t;
-    }                                 
+    }
     roulette[size(*pop) - 1] = RAND_MAX; //end of preparing roulette
-    for (vector<int>::iterator i = next.begin(); i != next.end(); i++) 
-        *i = lower_bound (roulette.begin(), roulette.end(), 
+    for (vector<int>::iterator i = next.begin(); i != next.end(); i++)
+        *i = lower_bound (roulette.begin(), roulette.end(),
                          static_cast<unsigned int>(rand())) - roulette.begin();
 }
 
@@ -325,7 +325,7 @@ void GAfit::tournament_selection(vector<int>& next)
     }
 }
 
-vector<int>::iterator 
+vector<int>::iterator
 GAfit::SRS_and_DS_common (vector<int>& next)
 {
     vector<int>::iterator r = next.begin();
@@ -384,7 +384,7 @@ void GAfit::scale_score () //return value - are individuals varying?
 
     //scaling p -> q - p; p -> p / <p>
     fp q = max_in_window();
-    if (q < 0) 
+    if (q < 0)
         q = std_dev_based_q();
     q += linear_scaling_b;
     fp sum = 0;
@@ -395,7 +395,7 @@ void GAfit::scale_score () //return value - are individuals varying?
     if (sum == 0) //to avoid x/0
         return;
     fp avg_rev_sc = sum / pop->size();
-    for (vector<Individual>::iterator i = pop->begin(); i != pop->end(); i++) 
+    for (vector<Individual>::iterator i = pop->begin(); i != pop->end(); i++)
         i->norm_score = i->reversed_score / avg_rev_sc;
 }
 
@@ -410,7 +410,7 @@ fp GAfit::std_dev_based_q()
     fp avg_p = sum_p / pop->size();
     fp sq_sigma = avg_p2 - avg_p * avg_p;
     fp sigma = sq_sigma > 0 ? sqrt (sq_sigma) : 0;  //because of problem with
-    return linear_scaling_a * avg_p + linear_scaling_c * sigma; // sqrt(0) 
+    return linear_scaling_a * avg_p + linear_scaling_c * sigma; // sqrt(0)
 }
 
 fp GAfit::max_in_window ()
@@ -425,8 +425,8 @@ fp GAfit::max_in_window ()
     if (window_size > 0) {
         if (rank_scoring)
             return size(*pop) - 1;
-        else 
-            return *max_element (max_raw_history.begin(), 
+        else
+            return *max_element (max_raw_history.begin(),
                                  max_raw_history.begin() + window_size);
     }
     else
@@ -458,8 +458,8 @@ bool GAfit::termination_criteria_and_print_info (int iter)
         generations_sum += i->generation;
     }
     fp std_dev = sq_sum > 0 ? sqrt (sq_sum / pop->size()) : 0;
-    F->msg("Population #" + S(iter_nr) + ": best " + S(min) 
-                + ", avg " + S(avg) + ", worst " + S(tmp_max) 
+    F->msg("Population #" + S(iter_nr) + ": best " + S(min)
+                + ", avg " + S(avg) + ", worst " + S(tmp_max)
                 + ", std dev. " + S(std_dev));
     if (min < best_indiv.raw_score) {
         best_indiv = *ibest;
@@ -471,7 +471,7 @@ bool GAfit::termination_criteria_and_print_info (int iter)
     //checking stop conditions
     bool stop = false;
 
-    if (common_termination_criteria(iter)) 
+    if (common_termination_criteria(iter))
         stop = true;
     if (std_dev < std_dev_stop * avg) {
         F->msg("Standard deviation of results is small enough to stop");
