@@ -1,14 +1,14 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// Licence: GNU General Public License version 2
-// $Id: data.cpp 400 2008-03-03 14:51:40Z wojdyr $
+// Licence: GNU General Public License ver. 2+
+// $Id: data.cpp 503 2009-06-09 15:47:31Z wojdyr $
 
 #include "common.h"
-#include "data.h" 
+#include "data.h"
 //#include "ui.h"
 #include "numfuncs.h"
-#include "datatrans.h" 
-#include "settings.h" 
-#include "logic.h" 
+#include "datatrans.h"
+#include "settings.h"
+#include "logic.h"
 
 #include <math.h>
 #include <string.h>
@@ -18,6 +18,7 @@
 #include <algorithm>
 
 #include <xylib/xylib.h>
+#include <xylib/cache.h>
 
 using namespace std;
 
@@ -52,7 +53,7 @@ string Data::get_info() const
     if (!filename.empty())
         s += "\nFilename: " + filename;
     if (given_x != INT_MAX || given_y != INT_MAX || given_s != INT_MAX)
-        s += "\nColumns: " + (given_x != INT_MAX ? S(given_x) : S("_")) 
+        s += "\nColumns: " + (given_x != INT_MAX ? S(given_x) : S("_"))
                     + ", " + (given_y != INT_MAX ? S(given_y) : S("_"));
     if (given_s != INT_MAX)
         s += ", " + S(given_s);
@@ -65,7 +66,7 @@ string Data::get_info() const
 
 void Data::clear()
 {
-    filename = "";   
+    filename = "";
     title = "";
     given_x = given_y = given_s = INT_MAX;
     given_options.clear();
@@ -84,13 +85,13 @@ void Data::post_load()
     if (!has_sigma) {
         int dds = F->get_settings()->get_e("data-default-sigma");
         if (dds == 's') {
-            for (vector<Point>::iterator i = p.begin(); i < p.end(); i++) 
+            for (vector<Point>::iterator i = p.begin(); i < p.end(); i++)
                 i->sigma = i->y > 1. ? sqrt (i->y) : 1.;
             inf += " No explicit std. dev. Set as sqrt(y)";
         }
         else if (dds == '1') {
-            for (vector<Point>::iterator i = p.begin(); i < p.end(); i++) 
-                i->sigma = 1.; 
+            for (vector<Point>::iterator i = p.begin(); i < p.end(); i++)
+                i->sigma = 1.;
             inf += " No explicit std. dev. Set as equal 1.";
         }
         else
@@ -117,7 +118,7 @@ void Data::recompute_y_bounds() {
     }
 }
 
-int Data::load_arrays(const vector<fp> &x, const vector<fp> &y, 
+int Data::load_arrays(const vector<fp> &x, const vector<fp> &y,
                       const vector<fp> &sigma, const string &data_title)
 {
     size_t size = y.size();
@@ -125,7 +126,7 @@ int Data::load_arrays(const vector<fp> &x, const vector<fp> &y,
     assert(sigma.empty() || sigma.size() == size);
     clear();
     title = data_title;
-    if (sigma.empty()) 
+    if (sigma.empty())
         for (size_t i = 0; i < size; ++i)
             p.push_back (Point (x[i], y[i]));
     else {
@@ -145,7 +146,7 @@ void Data::revert()
         throw ExecuteError("Dataset can't be reverted, it was not loaded "
                            "from file");
     string old_title = title;
-    string old_filename = filename; 
+    string old_filename = filename;
     // this->filename should not be passed by ref to load_file(), because it's
     // cleared before being used
     load_file(old_filename, given_x, given_y, given_s,
@@ -199,18 +200,18 @@ void shirley_bg(vector<Point> &pp, bool remove)
     for (int iter = 0; iter < max_iter; ++iter) {
         vector<double> Y(n);
         for (int i = 0; i < n; ++i)
-            Y[i] = pp[i].y - B[i]; 
+            Y[i] = pp[i].y - B[i];
         for (int i = 1; i < n; ++i)
             PA[i] = PA[i-1] + (Y[i] + Y[i-1]) / 2 * (pp[i].x - pp[i-1].x);
         double rel_diff = old_A != 0. ? abs(PA[n-1] - old_A) / old_A : 1.;
-        if (rel_diff < max_rdiff) 
+        if (rel_diff < max_rdiff)
             break;
         old_A = PA[n-1];
         for (int i = 0; i < n; ++i)
             B[i] = ya + dy / PA[n-1] * PA[i];
     }
     if (remove)
-        for (int i = 0; i < n; ++i) 
+        for (int i = 0; i < n; ++i)
             pp[i].y -= B[i];
     else
         for (int i = 0; i < n; ++i)
@@ -223,7 +224,7 @@ void apply_operation(vector<Point> &pp, string const& op)
     assert (!op.empty());
     if (op == "sum_same_x")
         merge_same_x(pp, false);
-    else if (op == "avg_same_x") 
+    else if (op == "avg_same_x")
         merge_same_x(pp, true);
     else if (op == "shirley_bg")
         shirley_bg(pp, false);
@@ -251,10 +252,10 @@ void Data::load_data_sum(vector<Data const*> const& dd, string const& op)
     std::vector<Point> new_p;
     string new_title = dd[0]->get_title();
     string new_filename = dd.size() == 1 ? dd[0]->get_filename() : "";
-    for (vector<Data const*>::const_iterator i = dd.begin()+1; 
+    for (vector<Data const*>::const_iterator i = dd.begin()+1;
                                                           i != dd.end(); ++i)
         new_title += " + " + (*i)->get_title();
-    for (vector<Data const*>::const_iterator i = dd.begin(); 
+    for (vector<Data const*>::const_iterator i = dd.begin();
                                                           i != dd.end(); ++i) {
         vector<Point> const& points = (*i)->points();
         new_p.insert(new_p.end(), points.begin(), points.end());
@@ -293,7 +294,7 @@ void Data::add_one_point(double x, double y, double sigma)
         fp max_diff = 1e-4 * fabs(x_step);
         if (idx == 0 && fabs((p[1].x - p[0].x) - x_step) < max_diff)
             ; //nothing, the same step
-        else if (idx == size(p) - 1 
+        else if (idx == size(p) - 1
                         && fabs((p[idx].x - p[idx-1].x) - x_step) < max_diff)
             ; //nothing, the same step
         else
@@ -301,52 +302,47 @@ void Data::add_one_point(double x, double y, double sigma)
     }
 }
 
+int Data::count_blocks(string const& fn, vector<string> const& options)
+{
+    shared_ptr<const xylib::DataSet> xyds(xylib::cached_load_file(fn, options));
+    return xyds->get_block_count();
+}
+
+int Data::count_columns(string const& fn, vector<string> const& options,
+                               int first_block)
+{
+    shared_ptr<const xylib::DataSet> xyds(xylib::cached_load_file(fn, options));
+    return xyds->get_block(first_block)->get_column_count();
+}
+
 
 // for column indices, INT_MAX is used as not given
-void Data::load_file (string const& filename_, 
-                      int idx_x, int idx_y, int idx_s, 
+void Data::load_file (string const& fn,
+                      int idx_x, int idx_y, int idx_s,
                       vector<int> const& blocks,
                       vector<string> const& options)
 {
-    static xylib::DataSet *xyds = NULL;
-    static string xyds_path;
-    static vector<string> xyds_options;
-
-    if (filename_.empty())
+    if (fn.empty())
         return;
 
+    shared_ptr<const xylib::DataSet> xyds(xylib::cached_load_file(fn, options));
+
+    clear(); //removing previous file
+
+    vector<int> bb = blocks.empty() ? vector1(0) : blocks;
+
+    string block_name;
     try {
-        if (filename_ != xyds_path || options != xyds_options) {
-            string format_name;
-            vector<string> options_tail;
-            if (!options.empty()) {
-                format_name = options[0];
-                options_tail = vector<string>(options.begin()+1, options.end());
-            }
-            // if xylib::load_file() throws exception, we keep value of xyds
-            xylib::DataSet *new_xyds = xylib::load_file(filename_, format_name,
-                                                        options_tail);
-            assert(new_xyds);
-            delete xyds;
-            xyds = new_xyds;
-            xyds_path = filename_;
-            xyds_options = options;
-        }
-
-        clear(); //removing previous file
-
-        vector<int> bb = blocks.empty() ? vector1(0) : blocks;
-
         for (vector<int>::const_iterator b = bb.begin(); b != bb.end(); ++b) {
             assert(xyds);
             xylib::Block const* block = xyds->get_block(*b);
-            xylib::Column const& xcol 
+            xylib::Column const& xcol
                 = block->get_column(idx_x != INT_MAX ?  idx_x : 1);
-            xylib::Column const& ycol 
+            xylib::Column const& ycol
                 = block->get_column(idx_y != INT_MAX ?  idx_y : 2);
             int n = block->get_point_count();
             if (n < 5 && bb.size() == 1)
-                F->warn("Only " + S(p.size()) + " data points found in file.");
+                F->warn("Only " + S(n) + " data points found in file.");
 
             if (idx_s == INT_MAX) {
                 for (int i = 0; i < n; ++i) {
@@ -354,7 +350,7 @@ void Data::load_file (string const& filename_,
                 }
             }
             else {
-                xylib::Column const& scol 
+                xylib::Column const& scol
                     = block->get_column(idx_s != INT_MAX ?  idx_s : 2);
                 for (int i = 0; i < n; ++i) {
                     p.push_back(Point(xcol.get_value(i), ycol.get_value(i),
@@ -369,28 +365,39 @@ void Data::load_file (string const& filename_,
                     x_step = -x_step;
                 }
             }
+            if (!ycol.name.empty()) {
+                if (!block_name.empty())
+                    block_name += "/";
+                block_name += ycol.name;
+                if (!xcol.name.empty())
+                    block_name += "(" + xcol.name + ")";
+            }
         }
-
-        title = get_file_basename(filename_);
-        if (idx_x != INT_MAX && idx_y != INT_MAX)
-            title += ":" + S(idx_x) + ":" + S(idx_y);
-
-        if (x_step == 0) {
-            sort(p.begin(), p.end());
-            x_step = find_step();
-        }
-
-        filename = filename_;   
-        given_x = idx_x;
-        given_y = idx_y;
-        given_s = idx_s;
-        given_blocks = blocks;
-        given_options = options;
-
-        post_load();
     } catch (runtime_error const& e) {
         throw ExecuteError(e.what());
     }
+
+    if (!block_name.empty())
+        title = block_name;
+    else {
+        title = get_file_basename(fn);
+        if (idx_x != INT_MAX && idx_y != INT_MAX)
+            title += ":" + S(idx_x) + ":" + S(idx_y);
+    }
+
+    if (x_step == 0) {
+        sort(p.begin(), p.end());
+        x_step = find_step();
+    }
+
+    filename = fn;
+    given_x = idx_x;
+    given_y = idx_y;
+    given_s = idx_s;
+    given_blocks = blocks;
+    given_options = options;
+
+    post_load();
 }
 
 fp Data::get_y_at (fp x) const
@@ -405,24 +412,31 @@ fp Data::get_y_at (fp x) const
     return y1 + (y2 - y1) * (x - x1) / (x2 - x1);
 }
 
-void Data::transform(const string &s) 
+void Data::transform(const string &s)
 {
     p = transform_data(s, p);
     sort(p.begin(), p.end());
-    update_active_p();
+    after_transform();
 }
 
-void Data::update_active_p() 
+void Data::after_transform()
+{
+    x_step = find_step();
+    update_active_p();
+    recompute_y_bounds();
+}
+
+void Data::update_active_p()
     // pre: p.x sorted
     // post: active_p sorted
 {
     active_p.clear();
     for (int i = 0; i < size(p); i++)
-        if (p[i].is_active) 
+        if (p[i].is_active)
             active_p.push_back(i);
 }
 
-// does anyone need it ? 
+// does anyone need it ?
 /*
 int Data::auto_range (fp y_level, fp x_margin)
 {
@@ -433,7 +447,7 @@ int Data::auto_range (fp y_level, fp x_margin)
     }
     bool state = (p.begin()->y >= y_level);
     for (vector<Point>::iterator i = p.begin(); i < p.end(); i++) {
-        if (state == (i->y >= y_level)) 
+        if (state == (i->y >= y_level))
             i->is_active = state;
         else if (state && i->y < y_level) {
             i->is_active = false;
@@ -444,7 +458,7 @@ int Data::auto_range (fp y_level, fp x_margin)
             state = false;
         }
         else { // !state && i->y >= y_level
-            vector<Point>::iterator b =  lower_bound(p.begin(), p.end(), 
+            vector<Point>::iterator b =  lower_bound(p.begin(), p.end(),
                                                         Point(i->x - x_margin));
             for (vector<Point>::iterator j = b; j <= i; j++)
                 j->is_active = true;
@@ -457,7 +471,7 @@ int Data::auto_range (fp y_level, fp x_margin)
 */
 
 //FIXME to remove it or to leave it?
-string Data::range_as_string () const 
+string Data::range_as_string () const
 {
     if (active_p.empty()) {
         F->warn ("File not loaded or all points inactive.");
@@ -466,7 +480,7 @@ string Data::range_as_string () const
     vector<Point>::const_iterator old_p = p.begin() + active_p[0];
     fp left =  old_p->x;
     string s = "[" + S (left) + " : ";
-    for (vector<int>::const_iterator i = active_p.begin() + 1; 
+    for (vector<int>::const_iterator i = active_p.begin() + 1;
                                                     i != active_p.end(); i++) {
         if (p.begin() + *i != old_p + 1) {
             fp right = old_p->x;
@@ -481,7 +495,7 @@ string Data::range_as_string () const
 }
 
 ///check for fixed step
-fp Data::find_step() 
+fp Data::find_step()
 {
     const fp tiny_relat_diff = 1e-4;
     if (p.size() < 2)
@@ -496,18 +510,18 @@ fp Data::find_step()
         max_step = max (max_step, step);
     }
     fp avg = (max_step + min_step) / 2;
-    if ((max_step - min_step) < tiny_relat_diff * fabs(avg)) 
+    if ((max_step - min_step) < tiny_relat_diff * fabs(avg))
         return avg;
-    else 
+    else
         return 0.;
 }
-        
+
 
 int Data::get_lower_bound_ac (fp x) const
 {
     //pre: p.x is sorted, active_p is sorted
     int pit = lower_bound (p.begin(), p.end(), Point(x,0)) - p.begin();
-    return lower_bound (active_p.begin(), active_p.end(), pit) 
+    return lower_bound (active_p.begin(), active_p.end(), pit)
         - active_p.begin();
 }
 
@@ -515,7 +529,7 @@ int Data::get_upper_bound_ac (fp x) const
 {
     //pre: p.x is sorted, active_p is sorted
     int pit = upper_bound (p.begin(), p.end(), Point(x,0)) - p.begin();
-    return upper_bound (active_p.begin(), active_p.end(), pit) 
+    return upper_bound (active_p.begin(), active_p.end(), pit)
         - active_p.begin();
 }
 
