@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// Licence: GNU General Public License version 2
-// $Id: aplot.cpp 401 2008-03-05 13:50:23Z wojdyr $
+// Licence: GNU General Public License ver. 2+
+// $Id: aplot.cpp 510 2009-06-14 16:18:01Z wojdyr $
 
 /// In this file:
 ///  Auxiliary Plot, for displaying residuals, peak positions, etc. (AuxPlot)
@@ -16,7 +16,7 @@
 
 #include "aplot.h"
 #include "frame.h"
-#include "../sum.h"
+#include "../model.h"
 #include "../data.h"
 #include "../logic.h"
 
@@ -34,12 +34,12 @@ enum {
     ID_aux_m_tfont                 ,
     ID_aux_yz_fit                  ,
     ID_aux_yz_change               ,
-    ID_aux_yz_auto                  
+    ID_aux_yz_auto
 };
 
 
 //===============================================================
-//                           AuxPlot (auxiliary plot) 
+//                           AuxPlot (auxiliary plot)
 //===============================================================
 BEGIN_EVENT_TABLE (AuxPlot, FPlot)
     EVT_PAINT (           AuxPlot::OnPaint)
@@ -49,7 +49,6 @@ BEGIN_EVENT_TABLE (AuxPlot, FPlot)
     EVT_LEFT_UP (         AuxPlot::OnLeftUp)
     EVT_RIGHT_DOWN (      AuxPlot::OnRightDown)
     EVT_MIDDLE_DOWN (     AuxPlot::OnMiddleDown)
-    EVT_KEY_DOWN   (      AuxPlot::OnKeyDown)
     EVT_MENU_RANGE (ID_aux_plot0, ID_aux_plot0+10, AuxPlot::OnPopupPlot)
     EVT_MENU (ID_aux_plot_ctr, AuxPlot::OnPopupPlotCtr)
     EVT_MENU (ID_aux_revd, AuxPlot::OnPopupReversedDiff)
@@ -62,66 +61,66 @@ END_EVENT_TABLE()
 
 void AuxPlot::OnPaint(wxPaintEvent&)
 {
-    frame->draw_crosshair(-1, -1); 
+    frame->draw_crosshair(-1, -1);
     buffered_draw();
     vert_line_following_cursor(mat_redraw);//draw, if necessary, vertical lines
 }
 
-inline double sum_value(vector<Point>::const_iterator pt, Sum const* sum)
+inline double model_value(vector<Point>::const_iterator pt, Model const* model)
 {
-    return sum->value(pt->x);
+    return model->value(pt->x);
 }
 
-double diff_of_data_for_draw_data (vector<Point>::const_iterator i, 
-                                   Sum const* sum)
+double diff_of_data_for_draw_data (vector<Point>::const_iterator i,
+                                   Model const* model)
 {
-    return i->y - sum_value(i, sum);
+    return i->y - model_value(i, model);
 }
 
-double rdiff_of_data_for_draw_data (vector<Point>::const_iterator i, 
-                                    Sum const* sum)
+double rdiff_of_data_for_draw_data (vector<Point>::const_iterator i,
+                                    Model const* model)
 {
-    return sum_value(i, sum) - i->y;
+    return model_value(i, model) - i->y;
 }
 
-double diff_stddev_of_data_for_draw_data (vector<Point>::const_iterator i, 
-                                          Sum const* sum)
+double diff_stddev_of_data_for_draw_data (vector<Point>::const_iterator i,
+                                          Model const* model)
 {
-    return (i->y - sum_value(i, sum)) / i->sigma;
+    return (i->y - model_value(i, model)) / i->sigma;
 }
 
-double rdiff_stddev_of_data_for_draw_data (vector<Point>::const_iterator i, 
-                                           Sum const* sum)
+double rdiff_stddev_of_data_for_draw_data (vector<Point>::const_iterator i,
+                                           Model const* model)
 {
-    return (sum_value(i, sum) - i->y) / i->sigma;
+    return (model_value(i, model) - i->y) / i->sigma;
 }
 
-double diff_chi2_of_data_for_draw_data (vector<Point>::const_iterator i, 
-                                        Sum const* sum)
+double diff_chi2_of_data_for_draw_data (vector<Point>::const_iterator i,
+                                        Model const* model)
 {
-    double t = (i->y - sum_value(i, sum)) / i->sigma;
+    double t = (i->y - model_value(i, model)) / i->sigma;
     return t*t;
 }
 
-double diff_y_perc_of_data_for_draw_data (vector<Point>::const_iterator i, 
-                                          Sum const* sum)
+double diff_y_perc_of_data_for_draw_data (vector<Point>::const_iterator i,
+                                          Model const* model)
 {
-    return i->y ? (i->y - sum_value(i, sum)) / i->y * 100 : 0;
+    return i->y ? (i->y - model_value(i, model)) / i->y * 100 : 0;
 }
 
-double rdiff_y_perc_of_data_for_draw_data (vector<Point>::const_iterator i, 
-                                           Sum const* sum)
+double rdiff_y_perc_of_data_for_draw_data (vector<Point>::const_iterator i,
+                                           Model const* model)
 {
-    return i->y ? (sum_value(i, sum) - i->y) / i->y * 100 : 0;
+    return i->y ? (model_value(i, model) - i->y) / i->y * 100 : 0;
 }
 
 void AuxPlot::draw(wxDC &dc, bool monochrome)
 {
-    int pos = frame->get_focused_ds_index();
+    int pos = frame->get_focused_data_index();
     Data const* data = ftk->get_data(pos);
-    Sum const* sum = ftk->get_sum(pos);
+    Model const* model = ftk->get_model(pos);
     if (auto_zoom_y || fit_y_once) {
-        fit_y_zoom(data, sum);
+        fit_y_zoom(data, model);
         fit_y_once = false;
     }
     const int pixel_width = get_pixel_width(dc);
@@ -137,17 +136,17 @@ void AuxPlot::draw(wxDC &dc, bool monochrome)
     if (mark_peak_ctrs) {
         int ymax = pixel_height;
         std::vector<wxPoint> const& t = master->get_special_points();
-        for (vector<wxPoint>::const_iterator i = t.begin(); i != t.end(); i++) 
+        for (vector<wxPoint>::const_iterator i = t.begin(); i != t.end(); i++)
             dc.DrawLine(i->x, 0, i->x, ymax);
     }
 
-    if (kind == apk_empty || data->is_empty()) 
+    if (kind == apk_empty || data->is_empty())
         return;
 
     if (x_axis_visible) {
         int Y0 = ys.px(0.);
         dc.DrawLine (0, Y0, pixel_width, Y0);
-        if (kind == apk_diff) 
+        if (kind == apk_diff)
             draw_zoom_text(dc, !monochrome);
     }
     if (y_axis_visible) {
@@ -160,13 +159,13 @@ void AuxPlot::draw(wxDC &dc, bool monochrome)
         draw_ytics(dc, v, !monochrome);
     }
 
-    fp (*f)(vector<Point>::const_iterator, Sum const*) = 0;
+    fp (*f)(vector<Point>::const_iterator, Model const*) = 0;
     bool cummulative = false;
-    if (kind == apk_diff) 
-        f = reversed_diff ? rdiff_of_data_for_draw_data 
+    if (kind == apk_diff)
+        f = reversed_diff ? rdiff_of_data_for_draw_data
                           : diff_of_data_for_draw_data;
     else if (kind == apk_diff_stddev)
-        f = reversed_diff ? rdiff_stddev_of_data_for_draw_data 
+        f = reversed_diff ? rdiff_stddev_of_data_for_draw_data
                           : diff_stddev_of_data_for_draw_data;
     else if (kind == apk_diff_y_perc)
         f = reversed_diff ? rdiff_y_perc_of_data_for_draw_data
@@ -176,21 +175,21 @@ void AuxPlot::draw(wxDC &dc, bool monochrome)
         cummulative = true;
     }
     wxColour col = monochrome ? dc.GetPen().GetColour() : wxNullColour;
-    draw_data (dc, f, data, sum, col, col, 0, cummulative);
+    draw_data (dc, f, data, model, col, col, 0, cummulative);
 }
 
-/// print zoom info - how it compares to zoom of the master plot (e.g. "x3"), 
+/// print zoom info - how it compares to zoom of the master plot (e.g. "x3"),
 /// it makes sense only for apk_diff plot, when master plot is not logarithmic
 void AuxPlot::draw_zoom_text(wxDC& dc, bool set_pen)
 {
-    if (master->get_y_scale().logarithm) 
+    if (master->get_y_scale().logarithm)
         return;
     if (set_pen)
         dc.SetTextForeground(xAxisCol);
-    set_font(dc, *wxNORMAL_FONT);  
-    string s = "x" + S(y_zoom);  
+    set_font(dc, *wxNORMAL_FONT);
+    string s = "x" + S(y_zoom);
     wxCoord w, h;
-    dc.GetTextExtent (s2wx(s), &w, &h); 
+    dc.GetTextExtent (s2wx(s), &w, &h);
     dc.DrawText (s2wx(s), get_pixel_width(dc) - w - 2, 2);
 }
 
@@ -198,35 +197,35 @@ void AuxPlot::OnMouseMove(wxMouseEvent &event)
 {
     int X = event.GetX();
     vert_line_following_cursor(mat_move, X);
-    frame->set_status_coord_info(xs.val(X), ys.val(event.GetY()), true);
+    frame->set_status_coords(xs.val(X), ys.val(event.GetY()), pte_aux);
     if (X < move_plot_margin_width)
-        set_cursor(wxCURSOR_POINT_LEFT);
+        SetCursor(wxCURSOR_POINT_LEFT);
     else if (X > GetClientSize().GetWidth() - move_plot_margin_width)
-        set_cursor(wxCURSOR_POINT_RIGHT);
+        SetCursor(wxCURSOR_POINT_RIGHT);
     else {
         frame->draw_crosshair(X, -1);
-        set_cursor(wxCURSOR_CROSS);
+        SetCursor(wxCURSOR_CROSS);
     }
 }
 
 void AuxPlot::OnLeaveWindow (wxMouseEvent&)
 {
-    frame->set_status_text("", sbf_coord);
+    frame->clear_status_coords();
     frame->draw_crosshair(-1, -1);
 }
 
 bool AuxPlot::is_zoomable()
 {
-    return kind == apk_diff || kind == apk_diff_stddev 
+    return kind == apk_diff || kind == apk_diff_stddev
            || kind == apk_diff_y_perc || kind == apk_cum_chi2;
 }
 
 void AuxPlot::set_scale(int pixel_width, int pixel_height)
 {
-    // this functions depends on x scale in MainPlot 
-    // sometimes AuxPlot is redrawed before MainPlot, so we are updating 
+    // this functions depends on x scale in MainPlot
+    // sometimes AuxPlot is redrawed before MainPlot, so we are updating
     // horizontal scale in MainPlot
-    master->set_scale(pixel_width, 0); 
+    master->set_scale(pixel_width, 0);
     xs = master->get_x_scale();
 
     if (kind == apk_cum_chi2) {
@@ -237,8 +236,8 @@ void AuxPlot::set_scale(int pixel_width, int pixel_height)
     switch (kind) {
         case apk_empty:
             ys.scale = 1.; //y scale doesn't matter
-            break; 
-        case apk_diff: 
+            break;
+        case apk_diff:
             if (master->get_y_scale().logarithm)
                 ys.scale = y_zoom;
             else
@@ -253,14 +252,14 @@ void AuxPlot::set_scale(int pixel_width, int pixel_height)
     }
     ys.origin = - pixel_height / 2. / ys.scale;
 }
- 
+
 void AuxPlot::read_settings(wxConfigBase *cf)
 {
     wxString path = wxT("/AuxPlot_") + name;
     cf->SetPath(path);
     kind = static_cast <Aux_plot_kind_enum> (cf->Read (wxT("kind"), apk_diff));
-    mark_peak_ctrs = cfg_read_bool (cf, wxT("markCtr"), false);  
-    reversed_diff = cfg_read_bool (cf, wxT("reversedDiff"), false);  
+    mark_peak_ctrs = cfg_read_bool (cf, wxT("markCtr"), false);
+    reversed_diff = cfg_read_bool (cf, wxT("reversedDiff"), false);
     auto_zoom_y = false;
     line_between_points = cfg_read_bool(cf, wxT("line_between_points"), true);
     point_radius = cf->Read (wxT("point_radius"), 1);
@@ -282,7 +281,7 @@ void AuxPlot::read_settings(wxConfigBase *cf)
 void AuxPlot::save_settings(wxConfigBase *cf) const
 {
     cf->SetPath(wxT("/AuxPlot_") + name);
-    cf->Write (wxT("kind"), (int) kind); 
+    cf->Write (wxT("kind"), (int) kind);
     cf->Write (wxT("markCtr"), mark_peak_ctrs);
     cf->Write (wxT("reversedDiff"), reversed_diff);
     cf->Write (wxT("line_between_points"), line_between_points);
@@ -294,7 +293,7 @@ void AuxPlot::save_settings(wxConfigBase *cf) const
     // nothing here now
 
     cf->SetPath(wxT("../Colors"));
-    cfg_write_color(cf, wxT("bg"), backgroundCol); 
+    cfg_write_color(cf, wxT("bg"), backgroundCol);
     cfg_write_color(cf, wxT("active_data"), activeDataCol);
     cfg_write_color(cf, wxT("inactive_data"),inactiveDataCol);
     cf->SetPath(wxT(".."));
@@ -310,15 +309,15 @@ void AuxPlot::OnLeftDown (wxMouseEvent &event)
     }
     int X = event.GetPosition().x;
     // if mouse pointer is near to left or right border, move view
-    if (X < move_plot_margin_width) 
+    if (X < move_plot_margin_width)
         frame->scroll_view_horizontally(-0.33);  // <--
-    else if (X > GetClientSize().GetWidth() - move_plot_margin_width) 
+    else if (X > GetClientSize().GetWidth() - move_plot_margin_width)
         frame->scroll_view_horizontally(+0.33); // -->
     else {
         mouse_press_X = X;
         vert_line_following_cursor(mat_start, mouse_press_X+1, mouse_press_X);
-        set_cursor(wxCURSOR_SIZEWE);  
-        frame->set_status_text("Select x range and release button to zoom..."); 
+        SetCursor(wxCURSOR_SIZEWE);
+        frame->set_status_text("Select x range and release button to zoom...");
         CaptureMouse();
     }
 }
@@ -329,8 +328,8 @@ bool AuxPlot::cancel_mouse_left_press()
         vert_line_following_cursor(mat_stop);
         ReleaseMouse();
         mouse_press_X = INT_MIN;
-        set_cursor(wxCURSOR_CROSS);  
-        frame->set_status_text(""); 
+        SetCursor(wxCURSOR_CROSS);
+        frame->set_status_text("");
         return true;
     }
     else
@@ -361,19 +360,19 @@ void AuxPlot::OnRightDown (wxMouseEvent &event)
     //wxMenu *kind_menu = new wxMenu;
     popup_menu.AppendRadioItem(ID_aux_plot0+0, wxT("&empty"), wxT("nothing"));
     popup_menu.AppendRadioItem(ID_aux_plot0+1, wxT("&diff"), wxT("y_d - y_s"));
-    popup_menu.AppendRadioItem(ID_aux_plot0+2, wxT("&weighted diff"), 
+    popup_menu.AppendRadioItem(ID_aux_plot0+2, wxT("&weighted diff"),
                                wxT("(y_d - y_s) / sigma"));
-    popup_menu.AppendRadioItem(ID_aux_plot0+3, wxT("&proc diff"), 
+    popup_menu.AppendRadioItem(ID_aux_plot0+3, wxT("&proc diff"),
                                wxT("(y_d - y_s) / y_d [%]"));
-    popup_menu.AppendRadioItem(ID_aux_plot0+4, wxT("cumul. &chi2"), 
+    popup_menu.AppendRadioItem(ID_aux_plot0+4, wxT("cumul. &chi2"),
                                wxT("cumulative chi square"));
     popup_menu.Check(ID_aux_plot0+kind, true);
     popup_menu.AppendSeparator();
-    popup_menu.AppendCheckItem(ID_aux_revd, wxT("reversed diff"), 
+    popup_menu.AppendCheckItem(ID_aux_revd, wxT("reversed diff"),
                                wxT(""));
     popup_menu.Check(ID_aux_revd, reversed_diff);
     popup_menu.AppendSeparator();
-    popup_menu.AppendCheckItem(ID_aux_plot_ctr, wxT("show peak po&sitions"), 
+    popup_menu.AppendCheckItem(ID_aux_plot_ctr, wxT("show peak po&sitions"),
                                wxT("mark centers of peaks"));
     popup_menu.Check(ID_aux_plot_ctr, mark_peak_ctrs);
     popup_menu.AppendSeparator();
@@ -404,19 +403,6 @@ void AuxPlot::OnMiddleDown (wxMouseEvent&)
     frame->GViewAll();
 }
 
-void AuxPlot::OnKeyDown (wxKeyEvent& event)
-{
-    if (event.GetKeyCode() == WXK_ESCAPE) {
-        cancel_mouse_left_press();
-    }
-    else if (should_focus_input(event)) {
-        cancel_mouse_left_press();
-        frame->focus_input(event);
-    }
-    else
-        event.Skip();
-}
-
 void AuxPlot::OnPopupPlot (wxCommandEvent& event)
 {
     kind = static_cast<Aux_plot_kind_enum>(event.GetId()-ID_aux_plot0);
@@ -451,7 +437,7 @@ void AuxPlot::OnPopupColor (wxCommandEvent& event)
     }
     else if (n == ID_aux_c_axis)
         color = &xAxisCol;
-    else 
+    else
         return;
     if (change_color_dlg(*color)) {
         refresh();
@@ -460,9 +446,9 @@ void AuxPlot::OnPopupColor (wxCommandEvent& event)
 
 void AuxPlot::OnPopupYZoom (wxCommandEvent&)
 {
-    int r = wxGetNumberFromUser(wxT("Set zoom in y direction [%]"), 
-                                wxT(""), wxT(""), 
-                                static_cast<int>(y_zoom * 100 + 0.5), 
+    int r = wxGetNumberFromUser(wxT("Set zoom in y direction [%]"),
+                                wxT(""), wxT(""),
+                                static_cast<int>(y_zoom * 100 + 0.5),
                                 1, 10000000);
     if (r > 0)
         y_zoom = r / 100.;
@@ -476,7 +462,7 @@ void AuxPlot::OnPopupYZoomFit (wxCommandEvent&)
     refresh(false);
 }
 
-void AuxPlot::fit_y_zoom(Data const* data, Sum const* sum)
+void AuxPlot::fit_y_zoom(Data const* data, Model const* model)
 {
     if (!is_zoomable())
         return;
@@ -487,32 +473,32 @@ void AuxPlot::fit_y_zoom(Data const* data, Sum const* sum)
         return;
     int pixel_height = GetClientSize().GetHeight();
     switch (kind) { // setting y_zoom
-        case apk_diff: 
+        case apk_diff:
             {
-            y = get_max_abs_y(diff_of_data_for_draw_data, first, last, sum);
+            y = get_max_abs_y(diff_of_data_for_draw_data, first, last, model);
             Scale const& mys = master->get_y_scale();
-            y_zoom = fabs (pixel_height / (2 * y 
+            y_zoom = fabs (pixel_height / (2 * y
                                            * (mys.logarithm ? 1 : mys.scale)));
             fp order = pow (10, floor (log10(y_zoom)));
             y_zoom = floor(y_zoom / order) * order;
             }
             break;
         case apk_diff_stddev:
-            y = get_max_abs_y(diff_stddev_of_data_for_draw_data, 
-                              first, last, sum);
+            y = get_max_abs_y(diff_stddev_of_data_for_draw_data,
+                              first, last, model);
             y_zoom_base = pixel_height / (2. * y);
             y_zoom = 0.9;
             break;
         case apk_diff_y_perc:
-            y = get_max_abs_y(diff_y_perc_of_data_for_draw_data, 
-                              first, last, sum);
+            y = get_max_abs_y(diff_y_perc_of_data_for_draw_data,
+                              first, last, model);
             y_zoom_base = pixel_height / (2. * y);
             y_zoom = 0.9;
             break;
         case apk_cum_chi2:
             y = 0.;
-            for (vector<Point>::const_iterator i = first; i < last; i++) 
-                y += diff_chi2_of_data_for_draw_data(i, sum);
+            for (vector<Point>::const_iterator i = first; i < last; i++)
+                y += diff_chi2_of_data_for_draw_data(i, model);
             y_zoom_base = pixel_height / y;
             y_zoom = 0.9;
             break;
