@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// Licence: GNU General Public License version 2
-// $Id: app.h 399 2008-03-01 14:15:02Z wojdyr $
+// Licence: GNU General Public License ver. 2+
+// $Id: app.h 503 2009-06-09 15:47:31Z wojdyr $
 
 #ifndef FITYK__WX_APP__H__
 #define FITYK__WX_APP__H__
@@ -14,9 +14,9 @@ class FApp: public wxApp
 {
 public:
     // default config name
-    wxString conf_filename; 
+    std::string conf_filename;
     // alternative config name
-    wxString alt_conf_filename; 
+    std::string alt_conf_filename;
     // directory for (named by user) config files
     wxString config_dir;
 
@@ -32,5 +32,5 @@ wxString get_full_path_of_help_file (wxString const& name);
 
 DECLARE_APP(FApp)
 
-#endif 
+#endif
 
