@@ -1,9 +1,9 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// Licence: GNU General Public License version 2
-// $Id: dialogs.cpp 408 2008-03-15 20:10:34Z wojdyr $
+// Licence: GNU General Public License ver. 2+
+// $Id: dialogs.cpp 503 2009-06-09 15:47:31Z wojdyr $
 
 /// In this file:
-///  small dialogs:  SumHistoryDlg, FitRunDlg, DataExportDlg, AboutDlg,
+///  small dialogs:  SumHistoryDlg, FitRunDlg, DataExportDlg,
 ///                  MergePointsDlg
 
 #include <wx/wxprec.h>
@@ -15,22 +15,20 @@
 #endif
 
 #include <wx/statline.h>
-#include <boost/spirit/version.hpp> //SPIRIT_VERSION for AboutDlg
-#include <xylib/xylib.h> //get_version() for AboutDlg
+#include <wx/config.h>
 
 #include "dialogs.h"
 #include "frame.h"
 #include "../fit.h"
 #include "../ui.h"
 #include "../settings.h"
-#include "../datatrans.h" 
+#include "../datatrans.h"
 #include "../logic.h"
 #include "../data.h"
-#include "../sum.h" // get_ff_names()
+#include "../model.h" // get_ff_names()
 
 #include "img/up_arrow.xpm"
 #include "img/down_arrow.xpm"
-#include "img/fityk.xpm"
 
 using namespace std;
 
@@ -45,7 +43,7 @@ enum {
 
     ID_DED_RADIO                   ,
     ID_DED_INACT_CB                ,
-    ID_DED_TEXT                   
+    ID_DED_TEXT
 };
 
 
@@ -65,9 +63,9 @@ BEGIN_EVENT_TABLE(SumHistoryDlg, wxDialog)
 END_EVENT_TABLE()
 
 SumHistoryDlg::SumHistoryDlg (wxWindow* parent, wxWindowID id)
-    : wxDialog(parent, id, wxT("Parameters History"), 
-               wxDefaultPosition, wxDefaultSize, 
-               wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER), 
+    : wxDialog(parent, id, wxT("Parameters History"),
+               wxDefaultPosition, wxDefaultSize,
+               wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER),
       lc(0)
 {
     wxBoxSizer *top_sizer = new wxBoxSizer(wxVERTICAL);
@@ -80,25 +78,25 @@ SumHistoryDlg::SumHistoryDlg (wxWindow* parent, wxWindowID id)
     up_arrow = new wxBitmapButton (this, ID_SHIST_UP, wxBitmap (up_arrow_xpm));
     arrows_sizer->Add (up_arrow, 0);
     arrows_sizer->Add (10, 10, 1);
-    down_arrow = new wxBitmapButton (this, ID_SHIST_DOWN, 
+    down_arrow = new wxBitmapButton (this, ID_SHIST_DOWN,
                                      wxBitmap (down_arrow_xpm));
     arrows_sizer->Add (down_arrow, 0);
     hsizer->Add (arrows_sizer, 0, wxALIGN_CENTER);
     top_sizer->Add (hsizer, 1, wxEXPAND);
 
     wxBoxSizer *buttons_sizer = new wxBoxSizer(wxHORIZONTAL);
-    compute_wssr_button = new wxButton (this, ID_SHIST_CWSSR, 
+    compute_wssr_button = new wxButton (this, ID_SHIST_CWSSR,
                                         wxT("Compute WSSRs"));
     buttons_sizer->Add (compute_wssr_button, 0, wxALL, 5);
     buttons_sizer->Add (10, 10, 1);
-    buttons_sizer->Add (new wxStaticText(this, -1, wxT("View parameters:")), 
+    buttons_sizer->Add (new wxStaticText(this, -1, wxT("View parameters:")),
                         0, wxALL|wxALIGN_CENTER, 5);
     for (int i = 0; i < 4; i++)
         buttons_sizer->Add (new SpinCtrl(this, ID_SHIST_V + i, view[i],
                                          0, view_max, 40),
                             0, wxALL, 5);
     buttons_sizer->Add (10, 10, 1);
-    buttons_sizer->Add (new wxButton (this, wxID_CLOSE, wxT("&Close")), 
+    buttons_sizer->Add (new wxButton (this, wxID_CLOSE, wxT("&Close")),
                         0, wxALL, 5);
     top_sizer->Add (buttons_sizer, 0, wxALIGN_CENTER);
 
@@ -115,15 +113,15 @@ void SumHistoryDlg::initialize_lc()
     assert (view_max != -1);
     for (int i = 0; i < 4; i++)
         view[i] = min (i, view_max);
-    lc = new wxListCtrl (this, ID_SHIST_LC, 
-                         wxDefaultPosition, wxSize(450, 250), 
+    lc = new wxListCtrl (this, ID_SHIST_LC,
+                         wxDefaultPosition, wxSize(450, 250),
                          wxLC_REPORT|wxLC_SINGLE_SEL|wxLC_HRULES|wxLC_VRULES
                              |wxSIMPLE_BORDER);
     lc->InsertColumn(0, wxT("No."));
     lc->InsertColumn(1, wxT("parameters"));
     lc->InsertColumn(2, wxT("WSSR"));
     for (int i = 0; i < 4; i++)
-        lc->InsertColumn(3 + i, wxString::Format(wxT("par. %i"), view[i])); 
+        lc->InsertColumn(3 + i, wxString::Format(wxT("par. %i"), view[i]));
 
     FitMethodsContainer const* fmc = ftk->get_fit_container();
     for (int i = 0; i != fmc->get_param_history_size(); ++i) {
@@ -149,7 +147,7 @@ void SumHistoryDlg::update_selection()
 {
     FitMethodsContainer const* fmc = ftk->get_fit_container();
     int index = fmc->get_active_nr();
-    lc->SetItemState (index, wxLIST_STATE_SELECTED|wxLIST_STATE_FOCUSED, 
+    lc->SetItemState (index, wxLIST_STATE_SELECTED|wxLIST_STATE_FOCUSED,
                              wxLIST_STATE_SELECTED|wxLIST_STATE_FOCUSED);
     up_arrow->Enable (index != 0);
     down_arrow->Enable (index != fmc->get_param_history_size() - 1);
@@ -179,12 +177,12 @@ void SumHistoryDlg::OnComputeWssrButton (wxCommandEvent&)
 {
     FitMethodsContainer const* fmc = ftk->get_fit_container();
     vector<double> const orig = ftk->get_parameters();
-    vector<DataWithSum*> dsds = frame->get_selected_ds();
+    vector<DataAndModel*> dms = frame->get_selected_dms();
 
     for (int i = 0; i != fmc->get_param_history_size(); ++i) {
         vector<double> const& item = fmc->get_item(i);
         if (item.size() == orig.size()) {
-            double wssr = ftk->get_fit()->do_compute_wssr(item, dsds, true);
+            double wssr = ftk->get_fit()->do_compute_wssr(item, dms, true);
             lc->SetItem(i, 2, wxString::Format(wxT("%g"), wssr));
         }
     }
@@ -203,7 +201,7 @@ void SumHistoryDlg::OnActivatedItem (wxListEvent& event)
     update_selection();
 }
 
-void SumHistoryDlg::OnViewSpinCtrlUpdate (wxSpinEvent& event) 
+void SumHistoryDlg::OnViewSpinCtrlUpdate (wxSpinEvent& event)
 {
     int v = event.GetId() - ID_SHIST_V;
     assert (0 <= v && v < 4);
@@ -214,12 +212,12 @@ void SumHistoryDlg::OnViewSpinCtrlUpdate (wxSpinEvent& event)
     wxListItem li;
     li.SetMask (wxLIST_MASK_TEXT);
     li.SetText(wxString::Format(wxT("par. %i"), n));
-    lc->SetColumn(3 + v, li); 
+    lc->SetColumn(3 + v, li);
     //update data in wxListCtrl
     FitMethodsContainer const* fmc = ftk->get_fit_container();
     for (int i = 0; i != fmc->get_param_history_size(); ++i) {
         vector<double> const& item = fmc->get_item(i);
-        wxString s = n < size(item) ? wxString::Format(wxT("%g"), item[n]) 
+        wxString s = n < size(item) ? wxString::Format(wxT("%g"), item[n])
                                     : wxString();
         lc->SetItem(i, 3 + v, s);
     }
@@ -237,32 +235,32 @@ END_EVENT_TABLE()
 
 FitRunDlg::FitRunDlg(wxWindow* parent, wxWindowID id, bool initialize)
     : wxDialog(parent, id, wxT("fit functions to data"),
-               wxDefaultPosition, wxDefaultSize, 
-               wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER) 
+               wxDefaultPosition, wxDefaultSize,
+               wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER)
 {
     wxBoxSizer *top_sizer = new wxBoxSizer(wxVERTICAL);
-    wxArrayString ds_choices;
-    sel = frame->get_selected_ds_indices();
+    wxArrayString data_choices;
+    sel = frame->get_selected_data_indices();
     string a = S(sel.size()) + " selected dataset";
     if (sel.size() == 1)
         a += ": @" + S(sel[0]);
     else
         a += "s";
-    ds_choices.Add(s2wx(a)); 
-    ds_choices.Add(wxT("all datasets"));
-    ds_rb = new wxRadioBox(this, -1, wxT("fit..."), 
-                           wxDefaultPosition, wxDefaultSize,
-                           ds_choices, 1, wxRA_SPECIFY_COLS);
-    if (ftk->get_ds_count() == 1)
-        ds_rb->Enable(1, false);
-    top_sizer->Add(ds_rb, 0, wxALL|wxEXPAND, 5);
+    data_choices.Add(s2wx(a));
+    data_choices.Add(wxT("all datasets"));
+    data_rb = new wxRadioBox(this, -1, wxT("fit..."),
+                             wxDefaultPosition, wxDefaultSize,
+                             data_choices, 1, wxRA_SPECIFY_COLS);
+    if (ftk->get_dm_count() == 1)
+        data_rb->Enable(1, false);
+    top_sizer->Add(data_rb, 0, wxALL|wxEXPAND, 5);
     wxBoxSizer *method_sizer = new wxBoxSizer(wxHORIZONTAL);
-    method_sizer->Add(new wxStaticText(this, -1, wxT("method:")), 
+    method_sizer->Add(new wxStaticText(this, -1, wxT("method:")),
                       0, wxALL|wxALIGN_CENTER_VERTICAL, 5);
     wxArrayString m_choices;
-    m_choices.Add(wxT("Levenberg-Marquardt")); 
+    m_choices.Add(wxT("Levenberg-Marquardt"));
     m_choices.Add(wxT("Nelder-Mead simplex"));
-    m_choices.Add(wxT("Genetic Algorithm")); 
+    m_choices.Add(wxT("Genetic Algorithm"));
     method_c = new wxChoice(this, -1, wxDefaultPosition, wxDefaultSize,
                             m_choices);
     int method_nr = ftk->get_settings()->get_e("fitting-method");
@@ -279,26 +277,26 @@ FitRunDlg::FitRunDlg(wxWindow* parent, wxWindowID id, bool initialize)
     max_sizer->Add(nomaxiter_st, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 5);
     max_sizer->Add(new wxStaticText(this, -1, wxT("max. WSSR evaluations")),
                    0, wxALL|wxALIGN_CENTER_VERTICAL|wxALIGN_RIGHT, 5);
-    int default_max_eval = ftk->get_settings()->get_i("max-wssr-evaluations"); 
+    int default_max_eval = ftk->get_settings()->get_i("max-wssr-evaluations");
     maxeval_sc = new SpinCtrl(this, -1, default_max_eval, 0, 999999, 70);
     max_sizer->Add(maxeval_sc, 0, wxALL, 5);
     nomaxeval_st = new wxStaticText(this, -1, wxT("(unlimited)"));
     max_sizer->Add(nomaxeval_st, 0, wxALIGN_CENTER_VERTICAL, 0);
     top_sizer->Add(max_sizer, 0);
-    
+
     initialize_cb = new wxCheckBox(this, -1, wxT("initialize method"));
     initialize_cb->SetValue(initialize);
     top_sizer->Add(initialize_cb, 0, wxALL, 5);
 
     bool autop = (ftk->get_settings()->getp("autoplot") == "on-fit-iteration");
-    autoplot_cb = new wxCheckBox(this, -1, 
+    autoplot_cb = new wxCheckBox(this, -1,
                                  wxT("refresh plot after each iteration"));
-    autoplot_cb->SetValue(autop); 
+    autoplot_cb->SetValue(autop);
     top_sizer->Add(autoplot_cb, 0, wxALL, 5);
 
 
     top_sizer->Add(new wxStaticLine(this, -1), 0, wxEXPAND|wxLEFT|wxRIGHT, 10);
-    top_sizer->Add(CreateButtonSizer(wxOK|wxCANCEL), 
+    top_sizer->Add(CreateButtonSizer(wxOK|wxCANCEL),
                    0, wxALL|wxALIGN_CENTER, 5);
     SetSizerAndFit(top_sizer);
     update_allow_continue();
@@ -312,14 +310,14 @@ void FitRunDlg::update_allow_continue()
     int m_sel = method_c->GetSelection();
     //use "::Fit", because Fit is also a method of wxDialog
     ::Fit const* f = ftk->get_fit_container()->get_method(m_sel);
-    if (ds_rb->GetSelection() == 0) {
-        vector<DataWithSum*> dsds(sel.size());
+    if (data_rb->GetSelection() == 0) {
+        vector<DataAndModel*> dms(sel.size());
         for (size_t i = 0; i < sel.size(); ++i)
-            dsds[i] = ftk->get_ds(sel[i]);
-        is_initialized = f->is_initialized(dsds);
+            dms[i] = ftk->get_dm(sel[i]);
+        is_initialized = f->is_initialized(dms);
     }
     else {
-        is_initialized = f->is_initialized(ftk->get_dsds());
+        is_initialized = f->is_initialized(ftk->get_dms());
     }
     initialize_cb->Enable(is_initialized);
 }
@@ -335,19 +333,19 @@ void FitRunDlg::OnOK(wxCommandEvent&)
     string cmd;
     int m = method_c->GetSelection();
     if (m != ftk->get_settings()->get_e("fitting-method"))
-        cmd += "with fitting-method=" 
+        cmd += "with fitting-method="
             + ftk->get_fit_container()->get_method(m)->name + " ";
 
     bool autop = (ftk->get_settings()->getp("autoplot") == "on-fit-iteration");
     if (autoplot_cb->GetValue() != autop) {
         cmd += string(cmd.empty() ? "with" : ",") + " autoplot=";
-        cmd += (autoplot_cb->GetValue() ? "on-fit-iteration " 
+        cmd += (autoplot_cb->GetValue() ? "on-fit-iteration "
                                         : "on-plot-change ");
     }
 
     int max_eval = maxeval_sc->GetValue();
-    if (max_eval != ftk->get_settings()->get_i("max-wssr-evaluations")) 
-        cmd += (cmd.empty() ? "with" : ",") 
+    if (max_eval != ftk->get_settings()->get_i("max-wssr-evaluations"))
+        cmd += (cmd.empty() ? "with" : ",")
                 + string(" max-wssr-evaluations=") + S(max_eval) + " ";
 
     bool ini = initialize_cb->GetValue();
@@ -358,7 +356,7 @@ void FitRunDlg::OnOK(wxCommandEvent&)
         cmd += S(max_iter);
 
     if (ini) {
-        if (ds_rb->GetSelection() == 0)
+        if (data_rb->GetSelection() == 0)
             cmd += frame->get_in_datasets();
         else
             cmd += " in @*";
@@ -376,18 +374,18 @@ bool export_data_dlg(wxWindow *parent, bool load_exported)
 {
     static wxString dir = wxConfig::Get()->Read(wxT("/exportDir"));
 
-    vector<int> sel = frame->get_selected_ds_indices();
-    int ds;
-    if (sel.size() == 1) 
-        ds = sel[0];
-    else if ((int) sel.size() == ftk->get_ds_count())
-        ds = -1;
+    vector<int> sel = frame->get_selected_data_indices();
+    int data_idx;
+    if (sel.size() == 1)
+        data_idx = sel[0];
+    else if ((int) sel.size() == ftk->get_dm_count())
+        data_idx = -1;
     else
         return false;
 
     string columns = "";
     if (!load_exported) {
-        DataExportDlg ded(parent, -1, ds);
+        DataExportDlg ded(parent, -1, data_idx);
         if (ded.ShowModal() != wxID_OK)
             return false;
         columns = " (" + ded.get_columns() + ")";
@@ -399,10 +397,10 @@ bool export_data_dlg(wxWindow *parent, bool load_exported)
     dir = fdlg.GetDirectory();
     if (fdlg.ShowModal() == wxID_OK) {
         string path = wx2s(fdlg.GetPath());
-        string ds_str = (ds == -1 ? string("@*") : "@" + S(ds));
-        ftk->exec("info " + ds_str + columns + " > '" + path + "'");
+        string dstr = (data_idx == -1 ? string("@*") : "@" + S(data_idx));
+        ftk->exec("info " + dstr + columns + " > '" + path + "'");
         if (load_exported)
-            ftk->exec(ds_str + " < '" + path + "'");
+            ftk->exec(dstr + " < '" + path + "'");
         return true;
     }
     else
@@ -418,17 +416,17 @@ BEGIN_EVENT_TABLE(DataExportDlg, wxDialog)
     EVT_TEXT(ID_DED_TEXT, DataExportDlg::OnTextChanged)
 END_EVENT_TABLE()
 
-DataExportDlg::DataExportDlg(wxWindow* parent, wxWindowID id, int ds_)
-    : wxDialog(parent, id, wxT("Export data/functions as points"), 
-               wxDefaultPosition, wxSize(600, 500), 
+DataExportDlg::DataExportDlg(wxWindow* parent, wxWindowID id, int data_idx)
+    : wxDialog(parent, id, wxT("Export data/functions as points"),
+               wxDefaultPosition, wxSize(600, 500),
                wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER),
-      ds(ds_)
+      data_idx_(data_idx)
 {
     wxBoxSizer *top_sizer = new wxBoxSizer(wxVERTICAL);
-    wxStaticText *st1 = new wxStaticText(this, -1, 
+    wxStaticText *st1 = new wxStaticText(this, -1,
                                          wxT("       Step 1: Select columns"));
     top_sizer->Add(st1, 0, wxTOP|wxLEFT|wxRIGHT, 5);
-    wxStaticText *st2 = new wxStaticText(this, -1, 
+    wxStaticText *st2 = new wxStaticText(this, -1,
                                          wxT("       Step 2: Choose a file"));
     st2->Enable(false);
     top_sizer->Add(st2, 0, wxALL, 5);
@@ -437,19 +435,20 @@ DataExportDlg::DataExportDlg(wxWindow* parent, wxWindowID id, int ds_)
     cv.Add(wxT("x, y"));
     choices.Add(wxT("x, y, sigma"));
     cv.Add(wxT("x, y, s"));
-    choices.Add(wxT("x, sum"));
+    choices.Add(wxT("x, model"));
     cv.Add(wxT("x, F(x)"));
-    choices.Add(wxT("x, sum, zero shift"));
+    choices.Add(wxT("x, model, x-correction"));
     cv.Add(wxT("x, F(x), Z(x)"));
-    choices.Add(wxT("x, y, sigma, sum"));
+    choices.Add(wxT("x, y, sigma, model"));
     wxString xysF = wxT("x, y, s, F(x)");
     cv.Add(xysF);
 
-    choices.Add(wxT("x, y, sigma, sum, all functions..."));
+    choices.Add(wxT("x, y, sigma, model, all functions..."));
     wxString all_func;
-    if (ds >= 0) {
-        vector<string> const& ff_names = ftk->get_sum(ds)->get_ff_names();
-        for (vector<string>::const_iterator i = ff_names.begin(); 
+    if (data_idx_ >= 0) {
+        vector<string> const& ff_names
+            = ftk->get_model(data_idx_)->get_ff_names();
+        for (vector<string>::const_iterator i = ff_names.begin();
                                                       i != ff_names.end(); ++i)
             all_func += wxT(", %") + s2wx(*i) + wxT("(x)");
         cv.Add(xysF + all_func);
@@ -457,7 +456,7 @@ DataExportDlg::DataExportDlg(wxWindow* parent, wxWindowID id, int ds_)
     else
         cv.Add(wxT(""));
 
-    choices.Add(wxT("x, y, sigma, sum, residual"));
+    choices.Add(wxT("x, y, sigma, model, residual"));
     cv.Add(wxT("x, y, s, F(x), y-F(x)"));
     choices.Add(wxT("x, y, sigma, weighted residual"));
     cv.Add(wxT("x, y, s, (y-F(x))/s"));
@@ -470,13 +469,13 @@ DataExportDlg::DataExportDlg(wxWindow* parent, wxWindowID id, int ds_)
             rb->Enable(i, false);
     }
     top_sizer->Add(rb, 0, wxALL|wxEXPAND, 5);
-    inactive_cb = new wxCheckBox(this, ID_DED_INACT_CB, 
+    inactive_cb = new wxCheckBox(this, ID_DED_INACT_CB,
                                  wxT("export also inactive points"));
     top_sizer->Add(inactive_cb, 0, wxALL, 5);
     text = new wxTextCtrl(this, ID_DED_TEXT, wxT(""));
     top_sizer->Add(text, 0, wxEXPAND|wxALL, 5);
     top_sizer->Add (new wxStaticLine(this, -1), 0, wxEXPAND|wxLEFT|wxRIGHT, 5);
-    top_sizer->Add(CreateButtonSizer(wxOK|wxCANCEL), 
+    top_sizer->Add(CreateButtonSizer(wxOK|wxCANCEL),
                    0, wxALL|wxALIGN_CENTER, 5);
     SetSizerAndFit(top_sizer);
 
@@ -509,8 +508,8 @@ void DataExportDlg::on_widget_change()
     inactive_cb->Enable(!custom);
 }
 
-void DataExportDlg::OnTextChanged(wxCommandEvent&) 
-{ 
+void DataExportDlg::OnTextChanged(wxCommandEvent&)
+{
     if (!text->IsModified())
         return;
     vector<string> cols = split_string(wx2s(text->GetValue()), ",");
@@ -528,88 +527,20 @@ void DataExportDlg::OnTextChanged(wxCommandEvent&)
     inactive_cb->SetValue(has_a);
 }
 
-void DataExportDlg::OnOk(wxCommandEvent& event) 
+void DataExportDlg::OnOk(wxCommandEvent& event)
 {
     // save current settings to config file
     wxConfigBase *config = wxConfig::Get();
     config->Write(wxT("/exportDataSel"), rb->GetSelection());
-    if (is_custom()) 
+    if (is_custom())
         config->Write(wxT("/exportDataText"), text->GetValue());
-    else 
+    else
         config->Write(wxT("/exportDataA"), inactive_cb->GetValue());
 
     event.Skip();
 }
 
 
-//===============================================================
-//                         AboutDlg   
-//===============================================================
-
-BEGIN_EVENT_TABLE (AboutDlg, wxDialog)
-    EVT_TEXT_URL (wxID_ANY, AboutDlg::OnTextURL)
-END_EVENT_TABLE()
-
-AboutDlg::AboutDlg(wxWindow* parent)
-    : wxDialog(parent, -1, wxT("About Fityk"), wxDefaultPosition, 
-               wxSize(350,400), wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER)
-{
-    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
-    sizer->Add(new wxStaticBitmap(this, -1, wxBitmap(fityk_xpm)),
-               0, wxALIGN_CENTER|wxALL, 5);
-    wxStaticText *name = new wxStaticText(this, -1, 
-                                          wxT("fityk ") + pchar2wx(VERSION));
-    name->SetFont(wxFont(18, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL,
-                         wxFONTWEIGHT_BOLD));
-    sizer->Add(name, 0, wxALIGN_CENTER|wxALL, 5); 
-    txt = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxSize(350,250), 
-                         wxTE_MULTILINE|wxTE_RICH2|wxNO_BORDER
-                             |wxTE_READONLY|wxTE_AUTO_URL);
-    txt->SetBackgroundColour(GetBackgroundColour());
-    txt->SetDefaultStyle(wxTextAttr(wxNullColour, wxNullColour, wxNullFont,
-                                    wxTEXT_ALIGNMENT_CENTRE));
-    
-    txt->AppendText(wxT("A curve fitting and data analysis program\n\n"));
-    txt->SetDefaultStyle(wxTextAttr(wxNullColour, wxNullColour, 
-                                    *wxITALIC_FONT));
-    txt->AppendText(wxT("powered by ") wxVERSION_STRING wxT("\n"));
-    txt->AppendText(wxString::Format(wxT("powered by Boost.Spirit %d.%d.%d\n"), 
-                                       SPIRIT_VERSION / 0x1000,
-                                       SPIRIT_VERSION % 0x1000 / 0x0100,
-                                       SPIRIT_VERSION % 0x0100));
-    txt->AppendText(wxT("powered by xylib ") + s2wx(xylib::get_version())
-                    + wxT("\n"));
-    txt->SetDefaultStyle(wxTextAttr(wxNullColour, wxNullColour, 
-                                    *wxNORMAL_FONT));
-    txt->AppendText(wxT("\nCopyright (C) 2001 - 2008 Marcin Wojdyr\n\n"));
-    txt->SetDefaultStyle(wxTextAttr(*wxBLUE));
-    txt->AppendText(wxT("http://www.unipress.waw.pl/fityk/\n\n"));
-    txt->SetDefaultStyle(wxTextAttr(*wxBLACK));
-    txt->AppendText(wxT("This program is free software; ")
-      wxT("you can redistribute it ")
-      wxT("and/or modify it under the terms of the GNU General Public ")
-      wxT("License, version 2, as published by the Free Software Foundation"));
-    sizer->Add (txt, 1, wxALL|wxEXPAND|wxFIXED_MINSIZE, 5);
-    //sizer->Add (new wxStaticLine(this, -1), 0, wxEXPAND|wxLEFT|wxRIGHT, 10);
-    wxButton *bu_ok = new wxButton (this, wxID_OK, wxT("OK"));
-    bu_ok->SetDefault();
-    sizer->Add (bu_ok, 0, wxALL|wxEXPAND, 10);
-    SetSizerAndFit(sizer);
-}
-
-void AboutDlg::OnTextURL(wxTextUrlEvent& event) 
-{
-    if (!event.GetMouseEvent().LeftDown()) {
-        event.Skip();
-        return;
-    }
-    long start = event.GetURLStart(),
-         end = event.GetURLEnd();
-    wxString url = txt->GetValue().Mid(start, end - start);
-    wxLaunchDefaultBrowser(url);
-}
-
-
 //===============================================================
 //                         MergePointsDlg
 //===============================================================
@@ -620,42 +551,42 @@ END_EVENT_TABLE()
 
 MergePointsDlg::MergePointsDlg(wxWindow* parent, wxWindowID id)
     : wxDialog(parent, id, wxT("Merge data points"),
-               wxDefaultPosition, wxDefaultSize, 
-               wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER) 
+               wxDefaultPosition, wxDefaultSize,
+               wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER)
 {
     wxBoxSizer *top_sizer = new wxBoxSizer(wxVERTICAL);
     inf = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize,
                          wxTE_RICH|wxTE_READONLY|wxTE_MULTILINE);
     update_info();
     top_sizer->Add(inf, 1, wxEXPAND|wxALL, 1);
-    wxBoxSizer *hsizer = new wxBoxSizer(wxHORIZONTAL); 
+    wxBoxSizer *hsizer = new wxBoxSizer(wxHORIZONTAL);
     dx_cb = new wxCheckBox(this, -1, wxT("merge points with |x1-x2|<"));
     dx_cb->SetValue(true);
     hsizer->Add(dx_cb, 0, wxALIGN_CENTER_VERTICAL);
     dx_val = new RealNumberCtrl(this, -1, ftk->get_settings()->getp("epsilon"));
     hsizer->Add(dx_val, 0);
     top_sizer->Add(hsizer, 0, wxALL, 5);
-    y_rb = new wxRadioBox(this, -1, wxT("set y as ..."), 
-                          wxDefaultPosition, wxDefaultSize, 
-                          make_wxArrStr(wxT("sum"), wxT("avg")),
+    y_rb = new wxRadioBox(this, -1, wxT("set y as ..."),
+                          wxDefaultPosition, wxDefaultSize,
+                          ArrayString(wxT("sum"), wxT("avg")),
                           1, wxRA_SPECIFY_ROWS);
     top_sizer->Add(y_rb, 0, wxEXPAND|wxALL, 5);
-    focused_data = frame->get_focused_ds_index();
-    wxString this_ds = wxString::Format(wxT("dataset @%d"), focused_data);
-    output_rb = new wxRadioBox(this, -1, wxT("output to ..."), 
-                               wxDefaultPosition, wxDefaultSize, 
-                               make_wxArrStr(this_ds, wxT("new dataset")),
+    focused_data = frame->get_focused_data_index();
+    wxString fdstr = wxString::Format(wxT("dataset @%d"), focused_data);
+    output_rb = new wxRadioBox(this, -1, wxT("output to ..."),
+                               wxDefaultPosition, wxDefaultSize,
+                               ArrayString(fdstr, wxT("new dataset")),
                                1, wxRA_SPECIFY_ROWS);
     top_sizer->Add(output_rb, 0, wxEXPAND|wxALL, 5);
     top_sizer->Add (new wxStaticLine(this, -1), 0, wxEXPAND|wxLEFT|wxRIGHT, 5);
-    top_sizer->Add(CreateButtonSizer(wxOK|wxCANCEL), 
+    top_sizer->Add(CreateButtonSizer(wxOK|wxCANCEL),
                    0, wxALL|wxALIGN_CENTER, 5);
     SetSizerAndFit(top_sizer);
 }
 
 void MergePointsDlg::update_info()
 {
-    vector<int> dd = frame->get_selected_ds_indices();
+    vector<int> dd = frame->get_selected_data_indices();
     Data const* data = ftk->get_data(dd[0]);
     fp x_min = data->get_x_min();
     fp x_max = data->get_x_max();
@@ -688,7 +619,7 @@ string MergePointsDlg::get_command()
     s += "@" + dat + " = ";
     if (dx_cb->GetValue())
         s += y_rb->GetSelection() == 0 ? "sum_same_x " : "avg_same_x ";
-    s += "@" + join_vector(frame->get_selected_ds_indices(), " + @");
+    s += "@" + join_vector(frame->get_selected_data_indices(), " + @");
     return s;
 }
 
