@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// Licence: GNU General Public License version 2
-// $Id: dload.cpp 406 2008-03-10 22:16:45Z wojdyr $
+// Licence: GNU General Public License ver. 2+
+// $Id: dload.cpp 503 2009-06-09 15:47:31Z wojdyr $
 
 /// In this file:
 ///  Custom Data Load Dialog (DLoadDlg) and helpers
@@ -20,11 +20,12 @@
 #include <wx/filename.h>
 
 #include <xylib/xylib.h>
+#include <xylib/cache.h>
 
-#include "dload.h" 
+#include "dload.h"
 #include "frame.h"  // frame->add_recent_data_file(get_filename())
 #include "plot.h" // scale_tics_step()
-#include "../logic.h" 
+#include "../logic.h"
 #include "../settings.h"
 #include "../data.h" // get_file_basename()
 #include "../common.h" //iround
@@ -42,7 +43,7 @@ enum {
     ID_DXLOAD_AUTO_PLOT           ,
     ID_DXLOAD_OPENHERE            ,
     ID_DXLOAD_OPENNEW             ,
-    ID_DXLOAD_FN                  
+    ID_DXLOAD_FN
 };
 
 
@@ -53,19 +54,18 @@ public:
 
     PreviewPlot(wxWindow* parent)
         : BufferedPanel(parent), block_nr(0), idx_x(1), idx_y(2),
-          data(NULL), data_updated(false)
+          data_updated(false)
         { backgroundCol = *wxBLACK; }
 
-    ~PreviewPlot() { delete data; }
     void OnPaint(wxPaintEvent &event);
     void draw(wxDC &dc, bool);
     void load_dataset(string const& filename, string const& filetype,
                       vector<string> const& options);
-    xylib::DataSet const* get_data() { return data_updated ? data : NULL; }
+    shared_ptr<const xylib::DataSet> get_data() const { return data; }
     void make_outdated() { data_updated = false; }
 
 private:
-    xylib::DataSet* data;
+    shared_ptr<const xylib::DataSet> data;
     bool data_updated; // if false, draw() doesn't do anything (plot is clear)
 
     double xScale, yScale;
@@ -87,14 +87,14 @@ void PreviewPlot::OnPaint(wxPaintEvent&)
 
 void PreviewPlot::draw(wxDC &dc, bool)
 {
-    if (!data || !data_updated 
+    if (data.get() == NULL || !data_updated
         || block_nr < 0 || block_nr >= data->get_block_count())
         return;
 
     xylib::Block const *block = data->get_block(block_nr);
 
-    if (idx_x < 0 || idx_x > block->get_column_count() 
-            || idx_y < 0 || idx_y > block->get_column_count()) 
+    if (idx_x < 0 || idx_x > block->get_column_count()
+            || idx_y < 0 || idx_y > block->get_column_count())
         return;
 
     xylib::Column const& xcol = block->get_column(idx_x);
@@ -111,15 +111,15 @@ void PreviewPlot::draw(wxDC &dc, bool)
     yScale = - (1 - 1.2 * margin) * H / dy;
     xOffset = - xcol.get_min() * xScale + margin * W;
     yOffset = H - ycol.get_min() * yScale - margin * H;
-    
+
     // draw scale
     dc.SetPen(*wxWHITE_PEN);
     dc.SetTextForeground(*wxWHITE);
-    dc.SetFont(*wxSMALL_FONT);  
+    dc.SetFont(*wxSMALL_FONT);
 
     // ... horizontal
     vector<double> minors;
-    vector<double> tics 
+    vector<double> tics
         = scale_tics_step(xcol.get_min(), xcol.get_max(np), 4, minors);
     for (vector<double>::const_iterator i = tics.begin(); i != tics.end(); ++i){
         int X = getX(*i);
@@ -159,30 +159,12 @@ void PreviewPlot::draw(wxDC &dc, bool)
 }
 
 
-// wrapper around xylib::load_file()
-// with added caching (if the same filename and options are given)
-void PreviewPlot::load_dataset(string const& filename, 
+void PreviewPlot::load_dataset(string const& filename,
                                string const& filetype,
                                vector<string> const& options)
 {
-    static string old_filename;
-    static string old_filetype;
-    static vector<string> old_options;
-    if (filename == old_filename && filetype == old_filetype 
-                                                && options == old_options) {
-        data_updated = true;
-        return;
-    }
-
     try {
-        // if xylib::load_file() throws exception, we keep value of data
-        xylib::DataSet *new_ds = xylib::load_file(filename, filetype, options);
-        assert(new_ds);
-        delete data;
-        data = new_ds;
-        old_filename = filename;
-        old_filetype = filetype;
-        old_options = options;
+        data = xylib::cached_load_file(filename, filetype, options);
         data_updated = true;
     } catch (runtime_error const& e) {
         data_updated = false;
@@ -208,8 +190,8 @@ END_EVENT_TABLE()
 /// n - data slot to be used by "Replace ..." button, -1 means none
 /// data - used for default settings (path, columns, etc.), not NULL
 DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
-    : wxDialog(parent, id, wxT("Data load (custom)"), 
-               wxDefaultPosition, wxDefaultSize, 
+    : wxDialog(parent, id, wxT("Data load (custom)"),
+               wxDefaultPosition, wxDefaultSize,
                wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER),
       data_nr(n), initialized(false)
 {
@@ -242,21 +224,21 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
     // ----- left panel -----
     dir_ctrl = new wxGenericDirCtrl(left_panel, -1, wxDirDialogDefaultFolderStr,
                        wxDefaultPosition, wxDefaultSize,
-// On MSW wxGenericDirCtrl with filteres vanishes 
+// On MSW wxGenericDirCtrl with filteres vanishes
 //#ifndef __WXMSW__
 //                       wxDIRCTRL_SHOW_FILTERS,
 //#else
                        0,
 //#endif
-                       wxT("All Files (*)|*|") 
+                       wxT("All Files (*)|*|")
                            + s2wx(xylib::get_wildcards_string()));
     left_sizer->Add(dir_ctrl, 1, wxALL|wxEXPAND, 5);
     wxFileName path = s2wx(data->get_filename());
     path.Normalize();
-    dir_ctrl->SetPath(path.GetFullPath()); 
+    dir_ctrl->SetPath(path.GetFullPath());
     filename_tc = new KFTextCtrl(left_panel, ID_DXLOAD_FN, path.GetFullPath());
     left_sizer->Add (filename_tc, 0, wxALL|wxEXPAND, 5);
-                                     
+
 
     // selecting block
     block_ch = new wxChoice(left_panel, ID_DXLOAD_BLOCK);
@@ -264,17 +246,17 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
 
     // selecting columns
     columns_panel = new wxPanel (left_panel, -1);
-    wxStaticBoxSizer *h2a_sizer = new wxStaticBoxSizer(wxHORIZONTAL, 
+    wxStaticBoxSizer *h2a_sizer = new wxStaticBoxSizer(wxHORIZONTAL,
                     columns_panel, wxT("Select columns (0 for point index):"));
-    h2a_sizer->Add (new wxStaticText (columns_panel, -1, wxT("x")), 
+    h2a_sizer->Add (new wxStaticText (columns_panel, -1, wxT("x")),
                     0, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
     x_column = new SpinCtrl (columns_panel, ID_DXLOAD_COLX, 1, 0, 99, 50);
     h2a_sizer->Add (x_column, 0, wxALL|wxALIGN_LEFT, 5);
-    h2a_sizer->Add (new wxStaticText (columns_panel, -1, wxT("y")), 
+    h2a_sizer->Add (new wxStaticText (columns_panel, -1, wxT("y")),
                     0, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
     y_column = new SpinCtrl (columns_panel, ID_DXLOAD_COLY, 2, 0, 99, 50);
     h2a_sizer->Add (y_column, 0, wxALL|wxALIGN_LEFT, 5);
-    std_dev_cb = new wxCheckBox(columns_panel, ID_DXLOAD_STDDEV_CB, 
+    std_dev_cb = new wxCheckBox(columns_panel, ID_DXLOAD_STDDEV_CB,
                                 wxT("std.dev."));
     std_dev_cb->SetValue(false);
     h2a_sizer->Add(std_dev_cb, 0, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL,5);
@@ -282,9 +264,9 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
     h2a_sizer->Add(s_column, 0, wxALL|wxALIGN_LEFT, 5);
     columns_panel->SetSizer(h2a_sizer);
     left_sizer->Add (columns_panel, 0, wxALL|wxEXPAND, 5);
-    if (data->get_given_x() != INT_MAX) 
+    if (data->get_given_x() != INT_MAX)
         x_column->SetValue(data->get_given_x());
-    if (data->get_given_y() != INT_MAX) 
+    if (data->get_given_y() != INT_MAX)
         y_column->SetValue(data->get_given_y());
     if (data->get_given_s() != INT_MAX) {
         std_dev_cb->SetValue(true);
@@ -292,29 +274,28 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
     }
 
     bool def_sqrt = (ftk->get_settings()->getp("data-default-sigma") == "sqrt");
-    sd_sqrt_cb = new wxCheckBox(left_panel, ID_DXLOAD_SDS, 
-                                wxT("set std. dev. as max(sqrt(y), 1.0)"));
+    sd_sqrt_cb = new wxCheckBox(left_panel, ID_DXLOAD_SDS,
+                                wxT("std. dev. = max(sqrt(y), 1)"));
     sd_sqrt_cb->SetValue(def_sqrt);
     left_sizer->Add (sd_sqrt_cb, 0, wxALL|wxEXPAND, 5);
 
-    wxStaticBoxSizer *dt_sizer = new wxStaticBoxSizer(wxVERTICAL, 
-                                    left_panel, wxT("Data title:"));
-    title_cb = new wxCheckBox(left_panel, ID_DXLOAD_HTITLE, 
-                              wxT("custom title"));
-    dt_sizer->Add(title_cb, 0, wxALL, 5);
+    wxBoxSizer *dt_sizer = new wxBoxSizer(wxHORIZONTAL);
+    title_cb = new wxCheckBox(left_panel, ID_DXLOAD_HTITLE,
+                              wxT("data title:"));
+    dt_sizer->Add(title_cb, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5);
     title_tc = new wxTextCtrl(left_panel, -1, wxT(""));
     title_tc->Enable(false);
-    dt_sizer->Add(title_tc, 0, wxALL|wxEXPAND, 5);
-    left_sizer->Add (dt_sizer, 0, wxALL|wxEXPAND, 5);
+    dt_sizer->Add(title_tc, 1, wxALL|wxALIGN_CENTER_VERTICAL, 5);
+    left_sizer->Add (dt_sizer, 0, wxEXPAND);
 
     StdDevCheckBoxChanged();
 
     // ----- right upper panel -----
-    text_preview =  new wxTextCtrl(rupper_panel, -1, wxT(""), 
+    text_preview =  new wxTextCtrl(rupper_panel, -1, wxT(""),
                                    wxDefaultPosition, wxDefaultSize,
                                    wxTE_RICH|wxTE_READONLY|wxTE_MULTILINE);
     rupper_sizer->Add(text_preview, 1, wxEXPAND|wxALL, 5);
-    auto_text_cb = new wxCheckBox(rupper_panel, ID_DXLOAD_AUTO_TEXT, 
+    auto_text_cb = new wxCheckBox(rupper_panel, ID_DXLOAD_AUTO_TEXT,
                                   wxT("view the first 64kB of file as text"));
     auto_text_cb->SetValue(false);
     rupper_sizer->Add(auto_text_cb, 0, wxALL, 5);
@@ -322,7 +303,7 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
     // ----- right bottom panel -----
     plot_preview = new PreviewPlot(rbottom_panel);
     rbottom_sizer->Add(plot_preview, 1, wxEXPAND|wxALL, 5);
-    auto_plot_cb = new wxCheckBox(rbottom_panel, ID_DXLOAD_AUTO_PLOT, 
+    auto_plot_cb = new wxCheckBox(rbottom_panel, ID_DXLOAD_AUTO_PLOT,
                                   wxT("plot"));
     auto_plot_cb->SetValue(true);
     rbottom_sizer->Add(auto_plot_cb, 0, wxALL, 5);
@@ -338,15 +319,15 @@ DLoadDlg::DLoadDlg (wxWindow* parent, wxWindowID id, int n, Data* data)
     top_sizer->Add (new wxStaticLine(this, -1), 0, wxEXPAND|wxLEFT|wxRIGHT, 5);
     wxBoxSizer *button_sizer = new wxBoxSizer(wxHORIZONTAL);
     string data_nr_str = (data_nr >= 0 ? S(data_nr) : S("?"));
-    open_here = new wxButton(this, ID_DXLOAD_OPENHERE, 
+    open_here = new wxButton(this, ID_DXLOAD_OPENHERE,
                                         wxT("&Replace @") + s2wx(data_nr_str));
     if (data_nr < 0)
         open_here->Enable(false);
-    open_new = new wxButton(this, ID_DXLOAD_OPENNEW, 
+    open_new = new wxButton(this, ID_DXLOAD_OPENNEW,
                             wxT("&Open in new slot"));
     button_sizer->Add(open_here, 0, wxALL, 5);
     button_sizer->Add(open_new, 0, wxALL, 5);
-    button_sizer->Add(new wxButton(this, wxID_CLOSE, wxT("&Close")), 
+    button_sizer->Add(new wxButton(this, wxID_CLOSE, wxT("&Close")),
                       0, wxALL, 5);
     top_sizer->Add(button_sizer, 0, wxALL|wxALIGN_CENTER, 0);
     initialized = true;
@@ -364,7 +345,7 @@ void DLoadDlg::StdDevCheckBoxChanged()
 
 void DLoadDlg::OnHTitleCheckBox (wxCommandEvent& event)
 {
-    if (!event.IsChecked()) 
+    if (!event.IsChecked())
         update_title_from_file();
     title_tc->Enable(event.IsChecked());
 }
@@ -372,7 +353,7 @@ void DLoadDlg::OnHTitleCheckBox (wxCommandEvent& event)
 void DLoadDlg::update_block_list()
 {
     vector<string> bb;
-    if (plot_preview && plot_preview->get_data())
+    if (plot_preview && plot_preview->get_data().get() != NULL)
         for (int i = 0; i < plot_preview->get_data()->get_block_count(); ++i) {
             const string& name = plot_preview->get_data()->get_block(i)->name;
             bb.push_back(name.empty() ? "Block #" + S(i+1) : name);
@@ -387,17 +368,18 @@ void DLoadDlg::update_block_list()
 
 void DLoadDlg::update_title_from_file()
 {
-    if (title_cb->GetValue()) 
+    if (title_cb->GetValue())
         return;
     string path = wx2s(dir_ctrl->GetFilePath());
     if (path.empty()) {
         title_tc->Clear();
         return;
     }
+    // TODO: use class Data for this
     string title = get_file_basename(path);
     int x = x_column->GetValue();
     int y = y_column->GetValue();
-    if (x != 1 || y != 2 || std_dev_cb->GetValue()) 
+    if (x != 1 || y != 2 || std_dev_cb->GetValue())
         title += ":" + S(x) + ":" + S(y);
     title_tc->SetValue(s2wx(title));
 }
@@ -413,7 +395,7 @@ void DLoadDlg::OnAutoTextCheckBox (wxCommandEvent& event)
 void DLoadDlg::OnAutoPlotCheckBox (wxCommandEvent& event)
 {
     update_plot_preview();
-    if (event.IsChecked()) 
+    if (event.IsChecked())
         update_title_from_file();
 }
 
@@ -424,7 +406,7 @@ void DLoadDlg::OnBlockChanged (wxCommandEvent&)
 
 void DLoadDlg::OnColumnChanged (wxSpinEvent&)
 {
-    if (auto_plot_cb->GetValue()) 
+    if (auto_plot_cb->GetValue())
         update_plot_preview();
     update_title_from_file();
 }
@@ -442,8 +424,8 @@ void DLoadDlg::OnOpenHere (wxCommandEvent&)
 
 void DLoadDlg::OnOpenNew (wxCommandEvent&)
 {
-    int d_nr = ftk->get_ds_count();
-    if (d_nr == 1 && !ftk->get_ds(0)->has_any_info())
+    int d_nr = ftk->get_dm_count();
+    if (d_nr == 1 && !ftk->get_dm(0)->has_any_info())
         d_nr = 0; // special case, @+ will not add new data slot
     ftk->exec(get_command("@+", d_nr));
     frame->add_recent_data_file(get_filename());
@@ -468,10 +450,10 @@ void DLoadDlg::update_plot_preview()
         plot_preview->idx_x = x_column->GetValue();
         plot_preview->idx_y = y_column->GetValue();
         plot_preview->block_nr = block_ch->GetSelection();
-        plot_preview->load_dataset(wx2s(dir_ctrl->GetFilePath()), 
+        plot_preview->load_dataset(wx2s(dir_ctrl->GetFilePath()),
                                    "", vector<string>());
     }
-    else 
+    else
         plot_preview->make_outdated();
     plot_preview->refresh();
 }
@@ -487,7 +469,7 @@ void DLoadDlg::on_path_change()
     if (auto_text_cb->GetValue())
         update_text_preview();
     block_ch->SetSelection(0);
-    if (auto_plot_cb->GetValue()) 
+    if (auto_plot_cb->GetValue())
         update_plot_preview();
     update_block_list();
     update_title_from_file();
